@@ -7,7 +7,8 @@
             [clj-time.core :as t]
             [puppetlabs.kitchensink.testutils :as testutils]
             [clojure.zip :as zip])
-  (:import (java.util ArrayList)))
+  (:import (java.util ArrayList)
+           (java.io ByteArrayInputStream)))
 
 (deftest array?-test
   (testing "array?"
@@ -416,7 +417,49 @@
 
     (testing "should produce the correct hash"
       (is (= "8843d7f92416211de9ebb963ff4ce28125932878"
-            (utf8-string->sha1 "foobar"))))))
+             (utf8-string->sha1 "foobar")))))
+
+  (testing "Computing a SHA-256 for a UTF-8 string"
+    (testing "should fail if not passed a string"
+      (is (thrown? AssertionError (utf8-string->sha256 1234))))
+
+    (testing "should produce a stable hash"
+      (is (= (utf8-string->sha256 "foobar")
+             (utf8-string->sha256 "foobar"))))
+
+    (testing "should produce the correct hash"
+      (is (= "c3ab8ff13720e8ad9047dd39466b3c8974e592c2fa383d4a3960714caef0c4f2"
+             (utf8-string->sha256 "foobar"))))))
+
+(deftest stream-hashing
+  (testing "Computing a SHA-256 hash for an input stream"
+    (testing "should fail if not passed an input stream"
+      (is (thrown? AssertionError (stream->sha256 "what"))))
+
+    (let [stream-fn #(ByteArrayInputStream. (.getBytes "foobar" "UTF-8"))]
+      (testing "should produce a stable hash"
+        (is (= (stream->sha256 (stream-fn))
+               (stream->sha256 (stream-fn)))))
+
+      (testing "should produce the correct hash"
+        (is (= "c3ab8ff13720e8ad9047dd39466b3c8974e592c2fa383d4a3960714caef0c4f2"
+               (stream->sha256 (stream-fn))))))))
+
+(deftest file-hashing
+  (testing "Computing a SHA-256 hash for a file"
+    (testing "should fail if not passed a file"
+      (is (thrown? AssertionError (file->sha256 "what"))))
+
+    (let [f (temp-file "sha256" ".txt")]
+      (spit f "foobar")
+
+      (testing "should produce a stable hash"
+        (is (= (file->sha256 f)
+               (file->sha256 f))))
+
+      (testing "should produce the correct hash"
+        (is (= "c3ab8ff13720e8ad9047dd39466b3c8974e592c2fa383d4a3960714caef0c4f2"
+               (file->sha256 f)))))))
 
 (deftest temp-file-name-test
   (testing "The file should not exist."
@@ -749,7 +792,7 @@
                (with-timeout 1 false
                  (wait-return 1005 true))))))))
 
-(deftest open-port-num-test
+(deftest ^:slow open-port-num-test
   (let [port-in-use (open-port-num)]
     (with-open [s (java.net.ServerSocket. port-in-use)]
       (let [open-ports (set (take 60000 (repeatedly open-port-num)))]
